import { Widget } from '@phosphor/widgets';
import { FileBrowser } from '@jupyterlab/filebrowser';
import { ObservableValue } from '@jupyterlab/observables';
import { GitHubDrive } from './contents';
/**
 * Widget for hosting the GitHub filebrowser.
 */
export declare class GitHubFileBrowser extends Widget {
    constructor(browser: FileBrowser, drive: GitHubDrive);
    /**
     * An editable widget hosting the current user name.
     */
    readonly userName: GitHubEditableName;
    /**
     * React to a change in user.
     */
    private _onUserChanged(sender, args);
    /**
     * React to the path changing for the browser.
     */
    private _onPathChanged();
    /**
     * React to a change in the validity of the drive.
     */
    private _updateErrorPanel();
    private _browser;
    private _drive;
    private _errorPanel;
    private _openGitHubButton;
    private _launchBinderButton;
    private _binderActive;
    private _changeGuard;
}
/**
 * A widget that hosts an editable field,
 * used to host the currently active GitHub
 * user name.
 */
export declare class GitHubEditableName extends Widget {
    constructor(initialName?: string, placeholder?: string);
    /**
     * The current name of the field.
     */
    readonly name: ObservableValue;
    private _pending;
    private _placeholder;
    private _nameNode;
    private _editNode;
}
/**
 * A widget hosting an error panel for the browser,
 * used if there is an invalid user name or if we
 * are being rate-limited.
 */
export declare class GitHubErrorPanel extends Widget {
    constructor(message: string);
}
