"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var algorithm_1 = require("@phosphor/algorithm");
var widgets_1 = require("@phosphor/widgets");
var apputils_1 = require("@jupyterlab/apputils");
var coreutils_1 = require("@jupyterlab/coreutils");
var observables_1 = require("@jupyterlab/observables");
var contents_1 = require("./contents");
/**
 * The base url for a mybinder deployment.
 */
var MY_BINDER_BASE_URL = 'https://mybinder.org/v2/gh';
/**
 * The GitHub base url.
 */
var GITHUB_BASE_URL = 'https://github.com';
/**
 * The className for disabling the mybinder button.
 */
var MY_BINDER_DISABLED = 'jp-MyBinderButton-disabled';
/**
 * Widget for hosting the GitHub filebrowser.
 */
var GitHubFileBrowser = /** @class */ (function (_super) {
    __extends(GitHubFileBrowser, _super);
    function GitHubFileBrowser(browser, drive) {
        var _this = _super.call(this) || this;
        _this._binderActive = false;
        _this._changeGuard = false;
        _this.addClass('jp-GitHubBrowser');
        _this.layout = new widgets_1.PanelLayout();
        _this.layout.addWidget(browser);
        _this._browser = browser;
        _this._drive = drive;
        // Create an editable name for the user/org name.
        _this.userName = new GitHubEditableName('', '<Edit User>');
        _this.userName.addClass('jp-GitHubEditableUserName');
        _this.userName.node.title = 'Click to edit user/organization';
        _this._browser.toolbar.addItem('user', _this.userName);
        _this.userName.name.changed.connect(_this._onUserChanged, _this);
        // Create a button that opens GitHub at the appropriate
        // repo+directory.
        _this._openGitHubButton = new apputils_1.ToolbarButton({
            onClick: function () {
                var url = GITHUB_BASE_URL;
                // If there is no valid user, open the GitHub homepage.
                if (!_this._drive.validUser) {
                    window.open(url);
                    return;
                }
                var localPath = _this._browser.model.manager.services.contents
                    .localPath(_this._browser.model.path);
                var resource = contents_1.parsePath(localPath);
                url = coreutils_1.URLExt.join(url, resource.user);
                if (resource.repository) {
                    url = coreutils_1.URLExt.join(url, resource.repository, 'tree', 'master', resource.path);
                }
                window.open(url);
            },
            className: 'jp-GitHubIcon',
            tooltip: 'Open this repository on GitHub'
        });
        _this._browser.toolbar.addItem('GitHub', _this._openGitHubButton);
        // Create a button the opens MyBinder to the appropriate repo.
        _this._launchBinderButton = new apputils_1.ToolbarButton({
            onClick: function () {
                // If binder is not active for this directory, do nothing.
                if (!_this._binderActive) {
                    return;
                }
                var localPath = _this._browser.model.manager.services.contents
                    .localPath(_this._browser.model.path);
                var resource = contents_1.parsePath(localPath);
                var url = coreutils_1.URLExt.join(MY_BINDER_BASE_URL, resource.user, resource.repository, 'master');
                // Attempt to open using the JupyterLab tree handler
                var tree = coreutils_1.URLExt.join('lab', 'tree', resource.path);
                window.open(url + ("?urlpath=" + tree));
            },
            tooltip: 'Launch this repository on mybinder.org',
            className: 'jp-MyBinderButton'
        });
        _this._browser.toolbar.addItem('binder', _this._launchBinderButton);
        // Set up a listener to check if we can launch mybinder.
        _this._browser.model.pathChanged.connect(_this._onPathChanged, _this);
        // Trigger an initial pathChanged to check for binder state.
        _this._onPathChanged();
        _this._drive.rateLimitedState.changed.connect(_this._updateErrorPanel, _this);
        return _this;
    }
    /**
     * React to a change in user.
     */
    GitHubFileBrowser.prototype._onUserChanged = function (sender, args) {
        var _this = this;
        if (this._changeGuard) {
            return;
        }
        this._changeGuard = true;
        this._browser.model.cd("/" + args.newValue).then(function () {
            _this._changeGuard = false;
            _this._updateErrorPanel();
            // Once we have the new listing, maybe give the file listing
            // focus. Once the input element is removed, the active element
            // appears to revert to document.body. If the user has subsequently
            // focused another element, don't focus the browser listing.
            if (document.activeElement === document.body) {
                var listing = _this._browser.layout.widgets[2];
                listing.node.focus();
            }
        });
    };
    /**
     * React to the path changing for the browser.
     */
    GitHubFileBrowser.prototype._onPathChanged = function () {
        var localPath = this._browser.model.manager.services.contents
            .localPath(this._browser.model.path);
        var resource = contents_1.parsePath(localPath);
        // If we are not already changing the user name, set it.
        if (!this._changeGuard) {
            this._changeGuard = true;
            this.userName.name.set(resource.user);
            this._changeGuard = false;
            this._updateErrorPanel();
        }
        // Check for a valid user.
        if (!this._drive.validUser) {
            this._launchBinderButton.addClass(MY_BINDER_DISABLED);
            this._binderActive = false;
            return;
        }
        // Check for a valid repo.
        if (!resource.repository) {
            this._launchBinderButton.addClass(MY_BINDER_DISABLED);
            this._binderActive = false;
            return;
        }
        // If we are in the root of the repository, check for one of
        // the special files indicating we can launch the repository on mybinder.
        // TODO: If the user navigates to a subdirectory without hitting the root
        // of the repository, we will not check for whether the repo is binder-able.
        // Figure out some way around this.
        if (resource.path === '') {
            var item = algorithm_1.find(this._browser.model.items(), function (i) {
                return i.name === 'requirements.txt' || i.name === 'environment.yml' ||
                    i.name === 'apt.txt' || i.name === 'REQUIRE' ||
                    i.name === 'Dockerfile' ||
                    (i.name === 'binder' && i.type === 'directory');
            });
            if (item) {
                this._launchBinderButton.removeClass(MY_BINDER_DISABLED);
                this._binderActive = true;
                return;
            }
            else {
                this._launchBinderButton.addClass(MY_BINDER_DISABLED);
                this._binderActive = false;
                return;
            }
        }
        // If we got this far, we are in a subdirectory of a valid
        // repository, and should not change the binderActive status.
        return;
    };
    /**
     * React to a change in the validity of the drive.
     */
    GitHubFileBrowser.prototype._updateErrorPanel = function () {
        var localPath = this._browser.model.manager.services.contents
            .localPath(this._browser.model.path);
        var resource = contents_1.parsePath(localPath);
        var rateLimited = this._drive.rateLimitedState.get();
        var validUser = this._drive.validUser;
        // If we currently have an error panel, remove it.
        if (this._errorPanel) {
            var listing = this._browser.layout.widgets[2];
            listing.node.removeChild(this._errorPanel.node);
            this._errorPanel.dispose();
            this._errorPanel = null;
        }
        // If we are being rate limited, make an error panel.
        if (rateLimited) {
            this._errorPanel = new GitHubErrorPanel('You have been rate limited by GitHub! ' +
                'You will need to wait about an hour before ' +
                'continuing');
            var listing = this._browser.layout.widgets[2];
            listing.node.appendChild(this._errorPanel.node);
            return;
        }
        // If we have an invalid user, make an error panel.
        if (!validUser) {
            var message = resource.user ?
                "\"" + resource.user + "\" appears to be an invalid user name!" :
                'Please enter a GitHub user name';
            this._errorPanel = new GitHubErrorPanel(message);
            var listing = this._browser.layout.widgets[2];
            listing.node.appendChild(this._errorPanel.node);
            return;
        }
    };
    return GitHubFileBrowser;
}(widgets_1.Widget));
exports.GitHubFileBrowser = GitHubFileBrowser;
/**
 * A widget that hosts an editable field,
 * used to host the currently active GitHub
 * user name.
 */
var GitHubEditableName = /** @class */ (function (_super) {
    __extends(GitHubEditableName, _super);
    function GitHubEditableName(initialName, placeholder) {
        if (initialName === void 0) { initialName = ''; }
        var _this = _super.call(this) || this;
        _this._pending = false;
        _this.addClass('jp-GitHubEditableName');
        _this._nameNode = document.createElement('div');
        _this._nameNode.className = 'jp-GitHubEditableName-display';
        _this._editNode = document.createElement('input');
        _this._editNode.className = 'jp-GitHubEditableName-input';
        _this._placeholder = placeholder || '<Edit Name>';
        _this.node.appendChild(_this._nameNode);
        _this.name = new observables_1.ObservableValue(initialName);
        _this._nameNode.textContent = initialName || _this._placeholder;
        _this.node.onclick = function () {
            if (_this._pending) {
                return;
            }
            _this._pending = true;
            Private.changeField(_this._nameNode, _this._editNode).then(function (value) {
                _this._pending = false;
                if (_this.name.get() !== value) {
                    _this.name.set(value);
                }
            });
        };
        _this.name.changed.connect(function (s, args) {
            if (args.oldValue !== args.newValue) {
                _this._nameNode.textContent =
                    args.newValue || _this._placeholder;
            }
        });
        return _this;
    }
    return GitHubEditableName;
}(widgets_1.Widget));
exports.GitHubEditableName = GitHubEditableName;
/**
 * A widget hosting an error panel for the browser,
 * used if there is an invalid user name or if we
 * are being rate-limited.
 */
var GitHubErrorPanel = /** @class */ (function (_super) {
    __extends(GitHubErrorPanel, _super);
    function GitHubErrorPanel(message) {
        var _this = _super.call(this) || this;
        _this.addClass('jp-GitHubErrorPanel');
        var image = document.createElement('div');
        var text = document.createElement('div');
        image.className = 'jp-GitHubErrorImage';
        text.className = 'jp-GitHubErrorText';
        text.textContent = message;
        _this.node.appendChild(image);
        _this.node.appendChild(text);
        return _this;
    }
    return GitHubErrorPanel;
}(widgets_1.Widget));
exports.GitHubErrorPanel = GitHubErrorPanel;
/**
 * A module-Private namespace.
 */
var Private;
(function (Private) {
    function changeField(text, edit) {
        // Replace the text node with an the input element.
        var parent = text.parentElement;
        var initialValue = text.textContent || '';
        edit.value = initialValue;
        parent.replaceChild(edit, text);
        edit.focus();
        // Highlight the input element
        var index = edit.value.lastIndexOf('.');
        if (index === -1) {
            edit.setSelectionRange(0, edit.value.length);
        }
        else {
            edit.setSelectionRange(0, index);
        }
        return new Promise(function (resolve, reject) {
            edit.onblur = function () {
                // Set the text content of the original node, then
                // replace the node.
                parent.replaceChild(text, edit);
                text.textContent = edit.value || initialValue;
                resolve(edit.value);
            };
            edit.onkeydown = function (event) {
                switch (event.keyCode) {
                    case 13:// Enter
                        event.stopPropagation();
                        event.preventDefault();
                        edit.blur();
                        break;
                    case 27:// Escape
                        event.stopPropagation();
                        event.preventDefault();
                        edit.value = initialValue;
                        edit.blur();
                        break;
                    default:
                        break;
                }
            };
        });
    }
    Private.changeField = changeField;
})(Private || (Private = {}));
